import os
import subprocess
import unittest
from unittest.mock import patch
import uuid

from datetime import date
from dateutil.relativedelta import relativedelta

import metabase
import trials

from .test_utils import db_exists


TMPFROZENDIR = os.path.expanduser("~/tmp/frozen")
FILESTOREDIR = os.path.expanduser("~/filestore")


def create_db(mode, status, **kwargs):
    dbuuid = str(uuid.uuid4())
    fs = os.path.join(FILESTOREDIR, dbuuid)
    subprocess.check_output(["mkdir", "-p", fs])
    subprocess.check_output(["createdb", dbuuid])
    metabase.add(
        dbuuid, 42152, "-1 hours", mode=mode, status=status, uuid=dbuuid, **kwargs
    )
    return metabase.get(dbuuid)


def fs_exists(dbname):
    return os.path.exists(os.path.join(FILESTOREDIR, dbname))


def meta_exists(dbname):
    return bool(metabase.get(dbname))


def frozen_archive_exists(dbname):
    return os.path.exists(
        os.path.join(TMPFROZENDIR, dbname + ".dmp")
    ) and os.path.exists(os.path.join(TMPFROZENDIR, dbname + ".filestore.tar.gz"))


class TestTrialGC(unittest.TestCase):
    @patch("trials.metabase.has_role")
    def test_to_activate_database_is_frozen(self, has_role):
        has_role.return_value = True

        db = create_db("trial", "to_activate")

        trials.gc()

        db = metabase.get(db.name)
        self.assertEqual(db.status, "frozen")
        self.assertEqual(db.expire_date.date(), date.today() + relativedelta(weeks=2))
        self.assertFalse(db_exists(db.name))
        self.assertFalse(fs_exists(db.name))
        self.assertTrue(meta_exists(db.name))
        self.assertTrue(frozen_archive_exists(db.name))

    def test_to_activate_duplicate_is_dropped_right_away(self):
        parent_db = create_db("trial", "activated")
        db = create_db("trial", "to_activate", source_uuid=parent_db.uuid)

        trials.gc()

        self.assertFalse(db_exists(db.name))
        self.assertFalse(fs_exists(db.name))
        self.assertFalse(meta_exists(db.name))
        self.assertFalse(frozen_archive_exists(db.name))

    @patch("trials.metabase.has_role")
    def test_frozen_archive_is_dropped(self, has_role):
        has_role.return_value = True

        db = create_db("trial", "to_activate")
        trials.gc()
        metabase.set_expiry_in(db.name, "-1 hours")

        trials.gc()

        self.assertFalse(db_exists(db.name))
        self.assertFalse(fs_exists(db.name))
        self.assertFalse(meta_exists(db.name))
        self.assertFalse(frozen_archive_exists(db.name))

    def test_activated_database_is_dropped_later(self):
        db = create_db("trial", "activated")

        trials.gc()

        db = metabase.get(db.name)
        self.assertEqual(db.mode, "drop")
        self.assertEqual(db.status, "unsubscribed")
        self.assertEqual(db.expire_date.date(), date.today() + relativedelta(weeks=3))
        self.assertTrue(db_exists(db.name))
        self.assertTrue(fs_exists(db.name))
        self.assertTrue(meta_exists(db.name))

    def test_activated_duplicate_is_dropped_right_away(self):
        parent_db = create_db("trial", "activated")
        db = create_db("trial", "activated", source_uuid=parent_db.uuid)

        trials.gc()

        self.assertFalse(db_exists(db.name))
        self.assertFalse(fs_exists(db.name))
        self.assertFalse(meta_exists(db.name))
